/***************************************************************************
Copyright (c) 2006 Mateusz Bilski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_BUFFER_H
#define SMART_TOOLS_BUFFER_H

#include "stsharedmemory.h"
#include "smarttools.h"
#include "stassert.h"
#include "ststring.h"
#include "ststringlist.h"

/*!
	\brief StBuffer represents a buffer.
	
	It uses the StSharedMemory class and enables basic operations on a buffer. 
	You can create a buffer from an StString or an StStringList.
	
	\include stbuffer.cpp
*/

class StBuffer
{
	public:
		/*! Create a zero-size buffer. */
		StBuffer() { StBuffer( 0 ); }
		
		/*! Create a buffer of given size. */
		StBuffer( unsigned long size ) { create( size ); }
		
		/*! Create a buffer from an StString. */
		StBuffer( const StString& str ) { createFrom( str ); }
		
		/*! Create a buffer from an StStringList. Join elements using endl. */
		StBuffer( const StStringList& list ) { createFrom( list ); }
		
		/*! Create a buffer the same as the given buffer. */
		StBuffer( const StBuffer& buffer ) { append( buffer ); }
		~StBuffer() { clean(); }
		
		/*! Create a buffer of given size. */
		bool create( unsigned long size ) { clean(); memory.malloc( size ); }
		
		/*! Create a buffer from an StString. */
		bool createFrom( const StString& str ) { create( str.length() + 1 ); memory=str.memory; }
		
		/*! Create a buffer from an StStringList. Join elements using endl. */
		bool createFrom( const StStringList& list ) { createFrom( list.join( "\n" ) ); }
		
		/*! Resize the buffer to a given size. */
		bool resize( unsigned long size ) { memory.realloc( size ); }
		
		/*! Add the given buffer to another buffer. */
		void append( const StBuffer& buffer );
		
		/*! Free memory. */
		void clean() { memory.free(); }
		
		/*! \return the full size of the buffer. */
		unsigned long size() const { return memory.getMemorySize(); }
		
		/*! \return a void pointer to the buffer. You can set the position, but it can't be bigger than the size. */
		void* get( unsigned long position = 0 ) const { ST_ASSERT( position <= size() ); return (char*)memory.getPointer()+position; }
	private:
		StSharedMemory memory;
};

#endif //SMART_TOOLS_BUFFER_H
